/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protegex.owl.model.visitor;

import edu.stanford.smi.protegex.owl.model.RDFResource;

import java.util.Collection;
import java.util.Iterator;

/**
 * User: matthewhorridge<br>
 * The Univeristy Of Manchester<br>
 * Medical Informatics Group<br>
 * Date: Jan 8, 2005<br><br>
 * <p/>
 * matthew.horridge@cs.man.ac.uk<br>
 * www.cs.man.ac.uk/~horridgm<br><br>
 */
public class VisitorUtil {

    /**
     * Visits the <code>RDFResource</code>s in the specified collection using the
     * specified visitor.
     * Note that this method does a type cast of everything in the collection
     * to <code>RDFResource</code>, so the specified collection should <b>only</b>
     * contain instances of <code>RDFResource</code>.
     *
     * @param collection A <code>Collection</code> that <b>only</b> contains instances
     *                   of <code>RDFResource</code>.
     */
    public static void visitRDFResources(Collection collection, OWLModelVisitor visitor) {
        for (Iterator it = collection.iterator(); it.hasNext();) {
            RDFResource curRes = (RDFResource) it.next();
            curRes.accept(visitor);
        }
    }
}

