/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */


package edu.stanford.smi.protegex.owl.swrl.bridge;

import edu.stanford.smi.protegex.owl.swrl.bridge.exceptions.DatatypeConversionException;
import edu.stanford.smi.protegex.owl.swrl.bridge.impl.ArgumentFactoryImpl;
import edu.stanford.smi.protegex.owl.swrl.bridge.xsd.PrimitiveXSDType;

import edu.stanford.smi.protegex.owl.swrl.bridge.xsd.PrimitiveXSDType;

import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFSLiteral;

import java.math.BigDecimal;
import java.math.BigInteger;

import java.util.*;

public abstract class ArgumentFactory
{
  public static ArgumentFactory getFactory() { return new ArgumentFactoryImpl(); }

  public abstract ClassArgument createClassArgument(String className);

  public abstract IndividualArgument createIndividualArgument(String individualName);

  public abstract ObjectPropertyArgument createObjectPropertyArgument(String propertyName);
  public abstract DatatypePropertyArgument createDatatypePropertyArgument(String propertyName);

  public abstract DatatypeValueArgument createDatatypeValueArgument(String s);
  public abstract DatatypeValueArgument createDatatypeValueArgument(Number n);
  public abstract DatatypeValueArgument createDatatypeValueArgument(boolean b);
  public abstract DatatypeValueArgument createDatatypeValueArgument(int i);
  public abstract DatatypeValueArgument createDatatypeValueArgument(long l);
  public abstract DatatypeValueArgument createDatatypeValueArgument(float f);
  public abstract DatatypeValueArgument createDatatypeValueArgument(double d);
  public abstract DatatypeValueArgument createDatatypeValueArgument(short s); 
  public abstract DatatypeValueArgument createDatatypeValueArgument(Byte b);
  public abstract DatatypeValueArgument createDatatypeValueArgument(BigDecimal bd);
  public abstract DatatypeValueArgument createDatatypeValueArgument(BigInteger bi);
  public abstract DatatypeValueArgument createDatatypeValueArgument(PrimitiveXSDType xsd);
  public abstract DatatypeValueArgument createDatatypeValueArgument(OWLModel owlModel, RDFSLiteral literal) throws DatatypeConversionException;

  public abstract MultiArgument createMultiArgument(String variableName, String prefixedVariableName);
  public abstract MultiArgument createMultiArgument(String variableName, String prefixedVariableName, List<BuiltInArgument> arguments);
} // ArgumentFactory
