/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */


package edu.stanford.smi.protegex.owl.swrl.bridge.impl;

import edu.stanford.smi.protegex.owl.swrl.bridge.*;
import edu.stanford.smi.protegex.owl.swrl.bridge.exceptions.DatatypeConversionException;
import edu.stanford.smi.protegex.owl.swrl.bridge.xsd.PrimitiveXSDType;

import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFSLiteral;

import java.math.BigDecimal;
import java.math.BigInteger;

import java.util.*;

public class ArgumentFactoryImpl extends ArgumentFactory
{
  public ClassArgument createClassArgument(String className) { return new OWLClassImpl(className); }

  public IndividualArgument createIndividualArgument(String individualName) { return new OWLIndividualImpl(individualName); }

  public ObjectPropertyArgument createObjectPropertyArgument(String propertyName) { return new OWLObjectPropertyImpl(propertyName); }
  public DatatypePropertyArgument createDatatypePropertyArgument(String propertyName) { return new OWLDatatypePropertyImpl(propertyName); }

  public DatatypeValueArgument createDatatypeValueArgument(String s) { return new OWLDatatypeValueImpl(s); }
  public DatatypeValueArgument createDatatypeValueArgument(Number n) { return new OWLDatatypeValueImpl(n); }
  public DatatypeValueArgument createDatatypeValueArgument(boolean b){ return new OWLDatatypeValueImpl(b); }
  public DatatypeValueArgument createDatatypeValueArgument(int i) { return new OWLDatatypeValueImpl(i); }
  public DatatypeValueArgument createDatatypeValueArgument(long l) { return new OWLDatatypeValueImpl(l); }
  public DatatypeValueArgument createDatatypeValueArgument(float f) { return new OWLDatatypeValueImpl(f); }
  public DatatypeValueArgument createDatatypeValueArgument(double d){ return new OWLDatatypeValueImpl(d); }
  public DatatypeValueArgument createDatatypeValueArgument(short s) { return new OWLDatatypeValueImpl(s); }
  public DatatypeValueArgument createDatatypeValueArgument(Byte b) { return new OWLDatatypeValueImpl(b); }
  public DatatypeValueArgument createDatatypeValueArgument(BigDecimal bd) { return new OWLDatatypeValueImpl(bd); }
  public DatatypeValueArgument createDatatypeValueArgument(BigInteger bi) { return new OWLDatatypeValueImpl(bi); }
  public DatatypeValueArgument createDatatypeValueArgument(PrimitiveXSDType xsd) { return new OWLDatatypeValueImpl(xsd); }
  public DatatypeValueArgument createDatatypeValueArgument(OWLModel owlModel, RDFSLiteral literal) throws DatatypeConversionException { return new OWLDatatypeValueImpl(owlModel, literal); }

  public MultiArgument createMultiArgument(String variableName, String prefixedVariableName) { return new MultiArgumentImpl(variableName, prefixedVariableName); }
  public MultiArgument createMultiArgument(String variableName, String prefixedVariableName, List<BuiltInArgument> arguments) { return new MultiArgumentImpl(variableName, prefixedVariableName, arguments); }
} // ArgumentFactory
